

import java.security.Key;
import java.util.Random;

import javax.crypto.Cipher;
import javax.crypto.KeyGenerator;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;

/**
 * This class enables the use of AES (Advanced Encryption Standard)
 * using the standard API of Java 1.6+.
 * <p>
 * This implementation will use AES with CBC and PKCS5Padding.
 * <p>
 * <ul>
 * <li>
 * CBD (Cipher Block Chaining Mode):
 * Is used to encrypt sequential blocks of data and make a chain of data that will
 * reduce probability to attack the AES Key by dictionary attack or brute force of small part
 * of encrypted data.
 * </li>
 * <li>
 * PKCS5Padding: Is a padding scheme to complete the final block, to make the block have the
 * right size, since a block of data need to have a fixed size based in the cipher, in this
 * case a block need to be multiple of 16 bytes (128 bits), due AES. A bad scheme for padding will create a easy way
 * to attack the final of the block data, where will be more easy to discover the key used to
 * encrypt.
 * </li>
 * </ul>
 * 
 * <p>
 * 
 * Copyright (C) MegaJogos. All Rights Reserved.
 * This can be used and redistributed for free for educational purpose only.
 * 
 * @author Graciliano M. P.
 * 
 */

public class AES {

	/**
	 * Generate the AES 128 bits key.
	 * <p>
	 * <i>Note that the use of 192 bits and 256 bits for AES may not be available on JRE due
	 * US restrictions to export cryptography softwares.</i>
	 *  
	 * @return
	 */
	static public Key generateKey() {
		try {
			KeyGenerator kgen = KeyGenerator.getInstance("AES");
			kgen.init(128); // 192 and 256 bits may not be available

			// Generate the secret key specs.
			return kgen.generateKey();
		}
		catch (Exception e) {
			e.printStackTrace() ;
			return null ;
		}

	}
	
	/**
	 * Encode the key into an array of bytes.
	 * <p>
	 * This should be used to send the key to another host.
	 * 
	 * @param key
	 * @return
	 */
	public static byte[] encodeKey(Key key) {
		return key.getEncoded() ;
	}
	
	/**
	 * Decode the array of bytes back to the Key instance.
	 * <p>
	 * Should be used when a key is received from another host. 
	 * 
	 * @param bytes
	 * @return
	 */
	public static Key decodeKey(byte[] bytes) {
		return new SecretKeySpec(bytes , "AES");
	}

	/**
	 * This is used when you encrypt/decrypt some data.
	 * <p>
	 * This is like a 2nd key to be used to encrypt/decrypt. 
	 * 
	 * @return
	 */
    static public byte[] createRandomIV() {
    	byte[] iv = new byte[16] ;
    	
    	Random rand = new Random() ;
    	
    	rand.nextBytes(iv) ;
    	
    	return iv ;
    }
    
    /**
     * Encrypt some data using AES and PKCS5Padding.
     * <p>
     * Note that CBC and PKCS5Padding is used to cipher block of data.
     * See class introduction for more about CBC and PKCS5Padding.     
     * 
     * @param key
     * @param data Array of bytes with the plain text.
     * @param iv Use <code>createRandomIV()</code> to generate it.
     * @return
     */
	static public byte[] encrypt(Key key, byte[] data , byte[] iv) {
		try {
			Cipher encrypt_cipher = Cipher.getInstance("AES/CBC/PKCS5Padding");
			encrypt_cipher.init(Cipher.ENCRYPT_MODE, key , new IvParameterSpec(iv) );
			
			return encrypt_cipher.doFinal( data ) ;
		}
		catch (Exception e) {
			e.printStackTrace() ;
			return null ;
		}
	}
	
	/**
	 * Decrypt data encrypted with AES, CBC and PKCS5Padding.
	 * 
	 * @param key
	 * @param encData Array of bytes with the encrypted data.
	 * @param iv Need to be the same used to <code>encrypt()</code>.
	 * @return
	 */
	static public byte[] decrypt(Key key , byte[] encData, byte[] iv) {
		try {
			Cipher decrypt_cipher = Cipher.getInstance("AES/CBC/PKCS5Padding");
			decrypt_cipher.init(Cipher.DECRYPT_MODE, key , new IvParameterSpec(iv));
			
			return decrypt_cipher.doFinal(encData);
		}
		catch (Exception e) {
			e.printStackTrace() ;
			return null ;
		}
	}
	
	///////////////////////////////////////////////////////////////////////////////////////
	
	public static void main(String[] args) {
		byte[] iv = AES.createRandomIV() ;
		
		String txt = "abcdefghijklmno" ;
		
		byte[] data = txt.getBytes() ;
		
		//data[15] = (byte)1 ;
		
		System.out.println(txt.length()+"<<"+txt+">>");
		
		// Generate a symmetric AES key:
		Key key = AES.generateKey() ;
		
		// Encrypt the data:
		byte[] encData = AES.encrypt( key , data , iv) ;
		
		String txt1 = new String(encData) ;
		System.out.println(txt1.length()+"<<"+txt1+">>");
		
		// Decrypt the data:
		byte[] decData = AES.decrypt( key , encData , iv) ;
		
		String txt2 = new String(decData) ;
		System.out.println(txt2.length()+"<<"+txt2+">>");
		
	}
	
	public static void main_(String[] args) {
		
		// Create random IV. Note that should be the same for encrypt and decrypt:
		byte[] iv = AES.createRandomIV() ;
		
		// The plain text:
		String txt = "abcdefghijklmnopqrstuvwxyz1234567890" ;
		
		// repeate the text:
		txt += txt ;
		txt += txt ;
		txt += txt ;
		txt += txt ;
		txt += txt ;
		

		// The bytes of the text:
		byte[] data = txt.getBytes() ;
		
		System.out.println(txt.length()+"<<"+txt+">>");
		
		// Generate a symmetric AES key:
		Key key = AES.generateKey() ;
		
		// Encrypt the data:
		byte[] encData = AES.encrypt( key , data , iv) ;
		
		String txt1 = new String(encData) ;
		System.out.println(txt1.length()+"<<"+txt1+">>");
		
		// Decrypt the data:
		byte[] decData = AES.decrypt( key , encData , iv) ;
		
		String txt2 = new String(decData) ;
		System.out.println(txt2.length()+"<<"+txt2+">>");
		
		// Check if decrypted data is equals original data:
		System.out.println("OK: "+ txt.equals(txt2) );
		
	}

}
