
import java.io.ByteArrayOutputStream;
import java.security.Key;
import java.security.KeyFactory;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.spec.PKCS8EncodedKeySpec;
import java.security.spec.X509EncodedKeySpec;

import javax.crypto.Cipher;

/**
 * This class enables the use of RSA using the standard API of Java 1.6+.
 * <p>
 * 
 * <i>Note that RSA is a strong and very slow algorithm! So RSA should be used to encrypt very
 * small amount of data, like a Key of a symmetric algorithm (like AES), than the symmetric
 * algorithm, that is faster than RSA, should be used to cipher the data.
 * </i>
 * <p>
 * 
 * Copyright (C) MegaJogos. All Rights Reserved.
 * This can be used and redistributed for free for educational purpose only.
 * 
 * @author Graciliano M. P.
 *
 */
public class RSA {
	
	/**
	 * Generate a RSA Key pair of 1024 bits.
	 * 
	 * @return KeyPair
	 */
	static public KeyPair generateKeyPair() {
		try {
			KeyPairGenerator generator = KeyPairGenerator.getInstance("RSA");
			generator.initialize(1024);
			KeyPair keypair = generator.generateKeyPair();
			
			return keypair ;			
		}
		catch (Exception e) {
			e.printStackTrace();
			return null ;
		}
	}

	/**
	 * Encode the Key (public or private) into an array of bytes.
	 * <p>
	 * This should be used to send the key to another host.
	 * 
	 * @param key
	 * @return
	 */
	public static byte[] encodeKey(Key key) {
		return key.getEncoded() ;
	}
	
	/**
	 * Decode the array of bytes back to the <b>public</b> Key instance.
	 * <p>
	 * Should be used when a key is received from another host. 
	 * 
	 * @param bytes
	 * @return
	 */
	public static PublicKey decodePublicKey(byte[] bytes) throws Exception {
		X509EncodedKeySpec pubspec = new X509EncodedKeySpec(bytes);
		KeyFactory factory = KeyFactory.getInstance("RSA");
		PublicKey pubkey = factory.generatePublic(pubspec);
		return pubkey;
	}
	
	
	/**
	 * Decode the array of bytes back to the <b>private</b> Key instance.
	 * <p>
	 * Should be used when a key is received from another host. 
	 * 
	 * @param bytes
	 * @return
	 */
	public static PrivateKey decodePrivateKey(byte[] bytes) throws Exception {
		PKCS8EncodedKeySpec privspec = new PKCS8EncodedKeySpec(bytes);
		KeyFactory factory = KeyFactory.getInstance("RSA");
		PrivateKey privkey = factory.generatePrivate(privspec);
		return privkey;
	}
	
	/**
	 * Encrypt data with RSA.
	 * <p>
	 * <i>Note that if you use the Private Key to encrypt you will need the Public Key to decrypt
	 * the encrypted data, and vice-versa.
	 * </i>
	 * 
	 * @param key
	 * @param data Array of bytes with the play text data.
	 * @return
	 */
	static public byte[] encrypt(Key key, byte[] data) {
		try {
			Cipher encrypt_cipher = Cipher.getInstance("RSA/ECB/PKCS1Padding");
			encrypt_cipher.init(Cipher.ENCRYPT_MODE, key);
			
			ByteArrayOutputStream bout = new ByteArrayOutputStream() ;
			
			for (int i = 0; i < data.length; i+=117) {
				int len = data.length - i ;
				if (len > 117) len = 117 ;
				byte[] enc = encrypt_cipher.doFinal( data , i , len );
				
				bout.write(enc) ;
			}
			
			return bout.toByteArray() ;	
		}
		catch (Exception e) {
			e.printStackTrace() ;
			return null ;
		}
	}
	
	/**
	 * Decrypt data encrypted with RSA. 
	 * <p>
	 * <i>Note that if you use the Private Key to encrypt you will need the Public Key to decrypt
	 * the encrypted data, and vice-versa.
	 * </i>
	 * 
	 * @param key
	 * @param encData Array of bytes with the encrypted data.
	 * @return
	 */
	static public byte[] decrypt(Key key , byte[] encData) {
		try {
			Cipher decrypt_cipher = Cipher.getInstance("RSA/ECB/PKCS1Padding");
			decrypt_cipher.init(Cipher.DECRYPT_MODE, key);
			
			ByteArrayOutputStream bout = new ByteArrayOutputStream() ;
			
			for (int i = 0; i < encData.length; i+=128) {
				int len = encData.length - i ;
				if (len > 128) len = 128 ;
				byte[] dec = decrypt_cipher.doFinal(encData , i , len);
				
				bout.write(dec) ;
			}

			return bout.toByteArray() ;
		}
		catch (Exception e) {
			e.printStackTrace() ;
			return null ;
		}
	}
	
	////////////////////////////////////////////////////////////////
	
	public static void main(String[] args) {
		
		KeyPair keyPair = RSA.generateKeyPair() ;
		
		// The plain text:
		String txt = "abcdefghijklmnopqrstuvwxyz1234567890" ;
		
		System.out.println(txt.length()+"<<"+txt+">>");
		
		// The bytes of the text:
		byte[] data = txt.getBytes() ;
		
		// Encrypt the data:
		byte[] encData = RSA.encrypt(keyPair.getPrivate() , data) ;
		
		String txt1 = new String(encData) ;
		System.out.println(txt1.length()+"<<"+txt1+">>");
		
		// Decrypt the data
		byte[] decData = RSA.decrypt(keyPair.getPublic(), encData) ;
		
		String txt2 = new String(decData) ;
		System.out.println(txt2.length()+"<<"+txt2+">>");
		
	}
	
   
}
